/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.wicket.jquery.ui;

import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.behavior.Behavior;
import org.apache.wicket.markup.html.IHeaderResponse;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.request.resource.PackageResourceReference;
import org.apache.wicket.request.resource.ResourceReference;

/**
 * Provides the base class for every jQuery behavior.
 * 
 * @author Sebastien Briquet - sebastien@7thweb.net
 *
 */
public abstract class JQueryAbstractBehavior extends Behavior
{
	private static final long serialVersionUID = 1L;
	private static final PackageResourceReference CORE_JS = new JavaScriptResourceReference(JQueryBehavior.class, "jquery-1.7.1.min.js"); 
	private static final PackageResourceReference CORE_UI = new JavaScriptResourceReference(JQueryBehavior.class, "jquery-ui-1.8.17.min.js");

	/**
	 * Behavior name
	 */
	private final String name;

	/**
	 * Additional references
	 */
	private final List<ResourceReference> references;
	
	/**
	 * Constructor.
	 * @param name the name of the behavior. It is used in the token so the behavior can be identified in the generated page.
	 */
	public JQueryAbstractBehavior(final String name)
	{
		this.name = name;
		this.references = new ArrayList<ResourceReference>();
	}

	
	/**
	 * Adds a reference to be added at {@link #renderHead(Component, IHeaderResponse)} time.
	 * @param reference
	 * @return true (as specified by Collection.add(E)) 
	 */
	protected boolean add(ResourceReference reference)
	{
		return this.references.add(reference);
	}	
	
	@Override
	public void renderHead(Component component, IHeaderResponse response)
	{
		response.renderJavaScriptReference(JQueryAbstractBehavior.CORE_JS);
		response.renderJavaScriptReference(JQueryAbstractBehavior.CORE_UI);
		
		for(ResourceReference reference : this.references)
		{
			if (reference instanceof JavaScriptResourceReference)
			{
				response.renderJavaScriptReference(reference);
			}
			
			if (reference instanceof CssResourceReference)
			{
				response.renderCSSReference(reference);
			}
		}

		response.renderJavaScript(this.$(), this.getToken());
	}
	
	/**
	 * Get the unique behavior token that act as the script id.
	 * @return the token
	 */
	String getToken()
	{
		return String.format("jquery-%s-%d", this.name, this.hashCode());
	}
	

	/**
	 * <code>
	 * http://blog.nemikor.com/2009/04/08/basic-usage-of-the-jquery-ui-dialog/
	 * </code>
	 */
	@Override
	public void beforeRender(Component component)
	{
		this.onBeforeRender();
		
		AjaxRequestTarget target = AjaxRequestTarget.get();
		
		if (target != null)
		{
			target.appendJavaScript(this.toString());
		}
	}
	
	/**
	 * Gets the jQuery statement.
	 * @return String like '$(function() { ... })'
	 */
	protected abstract String $();
	
	
	// Events //
	/**
	 * Triggered before render
	 */
	protected void onBeforeRender()
	{
		
	}


	@Override
	public String toString()
	{
		return this.$();
	}
}
