/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.wicket.jquery.ui.form.button;

import java.io.Serializable;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.markup.html.form.HiddenField;
import org.apache.wicket.markup.html.form.IFormSubmittingComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

import com.googlecode.wicket.jquery.ui.JQueryIcon;
import com.googlecode.wicket.jquery.ui.dialog.DialogButton;
import com.googlecode.wicket.jquery.ui.dialog.DialogIcon;
import com.googlecode.wicket.jquery.ui.dialog.MessageDialog;

/**
 * Provides a {@link AjaxButton} which pop-ups an OK-Cancel confirmation dialog when clicked. In case of confirmation, the form is sent via a http submit.<br/>
 * Note that this component is not an {@link AjaxButton} itself but a {@link FormComponentPanel}, it should not (and cannot) be attached to a &lt;button /&gt;; it can be attached on a &lt;div /&gt; or a &lt;span /&gt; for instance.
 *
 * @author Sebastien Briquet - sebastien@7thweb.net
 */
public abstract class ConfirmButton extends FormComponentPanel<String> implements IFormSubmittingComponent
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * If false, all standard processing like validating and model updating is skipped.
	 * @see IFormSubmittingComponent
	 */
	private boolean processForm = true;

	/**
	 * Constructor
	 * @param id markup id
	 * @param label the button text
	 * @param title the dialog title
	 * @param message the dialog message
	 */
	public ConfirmButton(String id, String label, String title, String message)
	{
		this(id, label, title, new Model<String>(message));
	}

	/**
	 * Constructor
	 * @param id markup id
	 * @param label the button text
	 * @param title the dialog title
	 * @param message the dialog message
	 */
	//TODO: 1.1 ConfirmButton: try to manage the IModel<String> so the message can be updated
	public ConfirmButton(String id, String label, String title, IModel<String> message)
	{
		super(id, message);
		
		this.add(new HiddenField<Serializable>("submitter", new Model<Serializable>()).add(AttributeModifier.replace("name", id))); // this is required in order for IFormSubmittingComponent to be retrieved while getting the .
		
		final MessageDialog dialog = new MessageDialog("dialog", title, this.getModel(), DialogButton.OK_CANCEL, DialogIcon.WARN) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onClose(AjaxRequestTarget target, String button)
			{
				if (BTN_OK.equals(button))
				{
					target.appendJavaScript(String.format("$('%s').submit();", JQueryWidget.getSelector(ConfirmButton.this.getForm()))); //not tested in nested form
				}
			}
		};

		this.add(dialog);

		final AjaxButton button = new AjaxButton("button") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected JQueryIcon getIcon()
			{
				return JQueryIcon.Alert;
			}

			@Override
			protected void onSubmit(AjaxRequestTarget target, Form<?> form)
			{
				dialog.open(target);
			}
		};
		
		this.add(button.setDefaultFormProcessing(false)); //does not validate the form before the dialog is being displayed
		
		button.add(new Label("label", new Model<String>(label)).setRenderBodyOnly(true));
	}

	// IFormSubmittingComponent //
	@Override
	public final boolean getDefaultFormProcessing()
	{
		return this.processForm;
	}
	
	@Override
	public final ConfirmButton setDefaultFormProcessing(boolean processForm)
	{
		this.processForm = processForm;
		
		return this;
	}
	
	@Override
	public abstract void onSubmit();
}
