/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.wicket.jquery.ui.form.slider;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.event.IEvent;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.FormComponent;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.model.IModel;

import com.googlecode.wicket.jquery.ui.JQueryBehavior;
import com.googlecode.wicket.jquery.ui.JQueryEvent;
import com.googlecode.wicket.jquery.ui.ajax.JQueryAjaxPostBehavior;

/**
 * Provides a jQuery date-picker based on a {@link FormComponentPanel}
 * This ajax version will post the {@link Component}, using a {@link JQueryAjaxPostBehavior}, when the 'change' javascript method is called.
 * 
 * TODO: post an issue: a page containing an AjaxSlider (FormComponent or FormComponentPanel could be the HomePage (Application#getHomePage()): the ServetWebRequest#getPostParameters() does not return the "slider:input" parameter.
 *  
 * @author Sebastien Briquet - sebastien@7thweb.net
 */
public class AjaxSlider extends Slider
{
	private static final long serialVersionUID = 1L;

	private JQueryAjaxPostBehavior changeBehavior;
	
	/**
	 * Constructor
	 * @param id the markup id
	 */
	public AjaxSlider(String id)
	{
		super(id);
	}
	
	/**
	 * Constructor
	 * @param id the markup id
	 * @param model the {@link IModel}
	 */
	public AjaxSlider(String id, IModel<Integer> model)
	{
		super(id, model);
	}
	
	/**
	 * Constructor
	 * @param id the markup id
	 * @param model the {@link IModel}
	 * @param label {@link Label} on which the current slide value will be displayed
	 */
	public AjaxSlider(String id, IModel<Integer> model, Label label)
	{
		super(id, model, label);
	}

	
	// Events //
	@Override
	protected void onInitialize()
	{
		super.onInitialize();  

		this.add(this.changeBehavior = this.newChangeBehavior(this, this.input));
	}

	@Override
	protected void onConfigure(JQueryBehavior behavior)
	{
		super.onConfigure();

		behavior.setOption("change", "function( event, ui ) { " + this.changeBehavior.getCallbackScript() + "}");
	}
	
	@Override
	public void onEvent(IEvent<?> event)
	{
		if (event.getPayload() instanceof AjaxSlider.ChangeEvent)
		{
			ChangeEvent payload = (ChangeEvent) event.getPayload();

			//In case of issue, consider copying code from AjaxFormComponentUpdatingBehavior.onEvent
			this.input.processInput();
			this.validate();
			this.onValueChanged(payload.getTarget(), this.getForm());
		}
	}

	
	/**
	 * Triggers when the value has changed
	 * @param target the {@link AjaxRequestTarget}
	 * @param form the {@link Form}
	 */
	protected void onValueChanged(AjaxRequestTarget target, Form<?> form)
	{
	}

	
	// Factories (Ajax behavior) //
	
	/**
	 * Gets a new {@link JQueryAjaxPostBehavior} that will be called on 'change' javascript method
	 * @param component the {@link FormComponent}
	 * @return the {@link JQueryAjaxPostBehavior}
	 */	
	private JQueryAjaxPostBehavior newChangeBehavior(AjaxSlider slider, FormComponent<?> component)
	{
		return new JQueryAjaxPostBehavior(slider, component) {
			
			private static final long serialVersionUID = 1L;

			@Override
			protected JQueryEvent newEvent(AjaxRequestTarget target)
			{
				return new ChangeEvent(target);
			}
		};
	}
	
	
	// Event class //
	/**
	 * Provides an event object that will be broadcasted by the {@link JQueryAjaxPostBehavior} 'change' callback
	 */
	public class ChangeEvent extends JQueryEvent
	{
		public ChangeEvent(AjaxRequestTarget target)
		{
			super(target);
		}
	}
}
