/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.googlecode.wicket.jquery.ui.form.slider;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.FormComponentPanel;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.IModel;
import org.apache.wicket.util.string.Strings;

import com.googlecode.wicket.jquery.ui.IJQueryWidget;
import com.googlecode.wicket.jquery.ui.JQueryBehavior;
import com.googlecode.wicket.jquery.ui.Options;

/**
 * Provides a jQuery date-picker based on a {@link FormComponentPanel}
 * 
 * @author Sebastien Briquet - sebastien@7thweb.net
 */
//TODO: 1.1 Slider: min & max
public class Slider extends FormComponentPanel<Integer> implements IJQueryWidget
{
	public enum Orientation {
		HORIZONTAL("'horizontal'"),		
		VERTICAL("'vertical'");
		
		private String orientation;
		
		private Orientation(String orientation)
		{
			this.orientation = orientation;
		}
		
		@Override
		public String toString()
		{
			return this.orientation;
		}
	}
	
	public enum Range {
		MIN("'min'"),		
		MAX("'max'");
		
		private String range;
		
		private Range(String range)
		{
			this.range = range;
		}
		
		@Override
		public String toString()
		{
			return this.range;
		}
	}
	
	private static final long serialVersionUID = 1L;
	private static final String METHOD = "slider";
	
	private Options options;
	protected TextField<Integer> input;
	private Label label; // the div on which the slider behavior will be applied to //
	private String labelId = null;

	/**
	 * Constructor
	 * @param id the markup id
	 */
	public Slider(String id)
	{
		super(id);
		this.init();
	}
	
	/**
	 * Constructor
	 * @param id the markup id
	 * @param model the {@link IModel}
	 */
	public Slider(String id, IModel<Integer> model)
	{
		super(id, model);
		this.init();
	}

	/**
	 * Constructor
	 * @param id the markup id
	 * @param model the {@link IModel}
	 * @param label {@link Label} on which the current slide value will be displayed
	 */
	public Slider(String id, IModel<Integer> model, Label label)
	{
		super(id, model);
		
		label.setDefaultModel(model);
		label.setOutputMarkupId(true);
		this.setLabelId(label.getMarkupId());
		
		this.init();
	}

//	public Slider(String id, IModel<Integer> minModel, IModel<Integer> maxModel)
//	{
//		this(id, maxModel);
//	}
	
	/**
	 * Initialization
	 */
	private void init()
	{
		this.options = new Options();

		this.input = new TextField<Integer>("input", this.getModel(), Integer.class);
		this.add(this.input.setOutputMarkupPlaceholderTag(true));

		this.label = new Label("slider", "");
		this.add(this.label);
	}
	
	@Override
 	protected void convertInput()
 	{
 		this.setConvertedInput(this.input.getConvertedInput());
 		this.updateModel(); //the TextField model has changed but this.onModelChanged() should be triggered. Another option would have been to override TextField.onModelChanged to call this.onModelChanged
 	}
	
	// Properties //
	/**
	 * Sets the label's markupId on which the current slide value will be displayed.<br/>
	 * <b>Warning:</b> It cannot be set/changed once the Component has been initialized.
	 * @param markupId the label's markupId
	 * @return this
	 */
	public Slider setLabelId(String markupId)
	{
		this.labelId = markupId;
		return this;
	}


	// Events //
	@Override
	protected void onInitialize()
	{
		super.onInitialize();
		
		this.add(JQueryWidget.newWidgetBehavior(this, this.label)); //cannot be in ctor as the markupId may be set manually afterward
	}

	/**
	 * Called immediately after the onConfigure method in a behavior. Since this is before the rendering 
	 * cycle has begun, the behavior can modify the configuration of the component (i.e. {@link Options})
	 * 
	 * @param behavior the {@link JQueryBehavior}
	 */
	protected void onConfigure(JQueryBehavior behavior)
	{
	}

	
	// IJQueryWidget //
	@Override
	public JQueryBehavior newWidgetBehavior(String selector)
	{
		return new JQueryBehavior(selector, METHOD) {
			
			private static final long serialVersionUID = 1L;

			@Override
			public void bind(Component component)
			{
				StringBuilder slideEvent = new StringBuilder();
				slideEvent.append("$('#").append(Slider.this.input.getMarkupId()).append("').val(ui.value); ");

				if (!Strings.isEmpty(Slider.this.labelId))
				{
					slideEvent.append("$('#").append(Slider.this.labelId).append("').text(ui.value); ");
				}

				this.setOption("slide", String.format("function(event, ui) { %s }", slideEvent));
			}

			@Override
			public void onConfigure(Component component)
			{
				Slider.this.onConfigure(this);

				this.setOption("value", Slider.this.getModelObject());
				this.setOptions(Slider.this.options);
			}
		};
	}
	
	/* Options */
	/**
	 * Sets the min value
	 * @param min
	 * @return {@link Slider} (this)
	 */
	public Slider setMin(Integer min)
	{
		this.options.set("min", min);
		return this;
	}
	
	/**
	 * Sets the max value
	 * @param max
	 * @return {@link Slider} (this)
	 */
	public Slider setMax(Integer max)
	{
		this.options.set("max", max);
		return this;
	}
	
	/**
	 * Sets the step value
	 * @param step
	 * @return {@link Slider} (this)
	 */
	public Slider setStep(Integer step)
	{
		this.options.set("step", step);
		return this;
	}
	
	/**
	 * Sets the {@link Range}
	 * @param range 
	 * @return {@link Slider} (this)
	 */
	public Slider setRange(Range range)
	{
		this.options.set("range", range);
		return this;
	}

	/**
	 * Sets the {@link Orientation} 
	 * @param orientation
	 * @return {@link Slider} (this)
	 */
	public Slider setOrientation(Orientation orientation)
	{
		this.options.set("orientation", orientation);
		return this;
	}
}
